@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU Library General Public
    License along with the Gnome Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301  USA.
 
    Author: Bastien Nocera <hadess@hadess.net>
  */
@@ -23,14 +23,10 @@
 
 #include <string.h>
 #include <glib.h>
-#include <libgnomevfs/gnome-vfs-mime-utils.h>
 
 #ifndef TOTEM_PL_PARSER_MINI
 #include <zlib.h>
 #include <gtk/gtk.h>
-#include <libgnomevfs/gnome-vfs.h>
-#include <libgnomevfs/gnome-vfs-mime.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include "xmlparser.h"
 #include "totem-pl-parser.h"
 #include "totemplparser-marshal.h"
@@ -239,20 +235,19 @@ parse_rss_items (TotemPlParser *parser, const char *url, xml_node_t *parent)
 
 TotemPlParserResult
 totem_pl_parser_add_rss (TotemPlParser *parser,
-			 const char *url,
-			 const char *base,
+			 GFile *file,
+			 GFile *base_file,
 			 gpointer data)
 {
 	xml_node_t* doc, *channel;
 	char *contents;
-	int size;
+	gsize size;
 
-	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	if (g_file_load_contents (file, NULL, &contents, &size, NULL, NULL) == FALSE)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	xml_parser_init (contents, size, XML_PARSER_CASE_INSENSITIVE);
 	if (xml_parser_build_tree_with_options (&doc, XML_PARSER_RELAXED | XML_PARSER_MULTI_TEXT) < 0) {
-		g_message ("can't parse the file");
 		g_free (contents);
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 	}
@@ -267,7 +262,12 @@ totem_pl_parser_add_rss (TotemPlParser *parser,
 
 	for (channel = doc->child; channel != NULL; channel = channel->next) {
 		if (g_ascii_strcasecmp (channel->name, "channel") == 0) {
+			char *url;
+
+			url = g_file_get_uri (file);
 			parse_rss_items (parser, url, channel);
+			g_free (url);
+
 			/* One channel per file */
 			break;
 		}
@@ -282,18 +282,23 @@ totem_pl_parser_add_rss (TotemPlParser *parser,
 /* http://www.apple.com/itunes/store/podcaststechspecs.html */
 TotemPlParserResult
 totem_pl_parser_add_itpc (TotemPlParser *parser,
-			  const char *url,
-			  const char *base,
+			  GFile *file,
+			  GFile *base_file,
 			  gpointer data)
 {
 	TotemPlParserResult ret;
 	char *new_url;
+	GFile *new_file;
 
-	new_url = g_strdup (url);
+	new_url = g_file_get_uri (file);
 	memcpy (new_url, "http", 4);
-	ret = totem_pl_parser_add_rss (parser, new_url, base, data);
+	new_file = g_file_new_for_uri (new_url);
 	g_free (new_url);
 
+	ret = totem_pl_parser_add_rss (parser, new_file, base_file, data);
+
+	g_object_unref (new_file);
+
 	return ret;
 }
 
@@ -429,15 +434,15 @@ parse_atom_entries (TotemPlParser *parser, const char *url, xml_node_t *parent)
 
 TotemPlParserResult
 totem_pl_parser_add_atom (TotemPlParser *parser,
-			  const char *url,
-			  const char *base,
+			  GFile *file,
+			  GFile *base_file,
 			  gpointer data)
 {
 	xml_node_t* doc;
-	char *contents;
-	int size;
+	char *contents, *url;
+	gsize size;
 
-	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	if (g_file_load_contents (file, NULL, &contents, &size, NULL, NULL) == FALSE)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	xml_parser_init (contents, size, XML_PARSER_CASE_INSENSITIVE);
@@ -453,7 +458,9 @@ totem_pl_parser_add_atom (TotemPlParser *parser,
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 	}
 
+	url = g_file_get_uri (file);
 	parse_atom_entries (parser, url, doc);
+	g_free (url);
 
 	g_free (contents);
 	xml_parser_free_tree (doc);
@@ -463,8 +470,8 @@ totem_pl_parser_add_atom (TotemPlParser *parser,
 
 TotemPlParserResult
 totem_pl_parser_add_xml_feed (TotemPlParser *parser,
-			      const char *url,
-			      const char *base,
+			      GFile *file,
+			      GFile *base_file,
 			      gpointer data)
 {
 	guint len;
@@ -475,11 +482,11 @@ totem_pl_parser_add_xml_feed (TotemPlParser *parser,
 	len = strlen (data);
 
 	if (totem_pl_parser_is_rss (data, len) != FALSE)
-		return totem_pl_parser_add_rss (parser, url, base, data);
+		return totem_pl_parser_add_rss (parser, file, base_file, data);
 	if (totem_pl_parser_is_atom (data, len) != FALSE)
-		return totem_pl_parser_add_atom (parser, url, base, data);
+		return totem_pl_parser_add_atom (parser, file, base_file, data);
 	if (totem_pl_parser_is_opml (data, len) != FALSE)
-		return totem_pl_parser_add_opml (parser, url, base, data);
+		return totem_pl_parser_add_opml (parser, file, base_file, data);
 
 	return TOTEM_PL_PARSER_RESULT_UNHANDLED;
 }
@@ -592,12 +599,12 @@ totem_pl_parser_parse_itms_doc (xml_node_t *item)
 	return NULL;
 }
 
-static char *
+static GFile *
 totem_pl_parser_get_feed_url (const char *data, gsize len)
 {
 	xml_node_t* doc;
 	const char *url;
-	char *ret;
+	GFile *ret;
 
 	url = NULL;
 
@@ -618,7 +625,7 @@ totem_pl_parser_get_feed_url (const char *data, gsize len)
 		return NULL;
 	}
 
-	ret = g_strdup (url);
+	ret = g_file_new_for_uri (url);
 	xml_parser_free_tree (doc);
 
 	return ret;
@@ -648,17 +655,18 @@ totem_pl_parser_get_itms_url (const char *data)
 
 TotemPlParserResult
 totem_pl_parser_add_itms (TotemPlParser *parser,
-			  const char *url,
-			  const char *base,
+			  GFile *file,
+			  GFile *base_file,
 			  gpointer data)
 {
-	char *contents, *uncompressed, *itms_url, *feed_url;
+	char *contents, *uncompressed, *itms_url;
+	GFile *itms_file, *feed_url;
 	TotemPlParserResult ret;
-	int size;
+	gsize size;
 
-	if (g_str_has_prefix (url, "itms") == FALSE) {
+	if (g_file_has_uri_scheme (file, "itms") == FALSE) {
 		/* Get the webpage */
-		if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+		if (g_file_load_contents (file, NULL, &contents, &size, NULL, NULL) == FALSE)
 			return TOTEM_PL_PARSER_RESULT_ERROR;
 
 		uncompressed = decompress_gzip (contents, size);
@@ -670,16 +678,19 @@ totem_pl_parser_add_itms (TotemPlParser *parser,
 		itms_url = totem_pl_parser_get_itms_url (uncompressed);
 		g_free (uncompressed);
 	} else {
-		itms_url= g_strdup (url);
+		itms_url= g_file_get_uri (file);
 		memcpy (itms_url, "http", 4);
 	}
 
 	/* Get the phobos linked, in some weird iTunes only format */
-	if (gnome_vfs_read_entire_file (itms_url, &size, &contents) != GNOME_VFS_OK) {
-		g_free (itms_url);
+	itms_file = g_file_new_for_uri (itms_url);
+	g_free (itms_url);
+
+	if (g_file_load_contents (itms_file, NULL, &contents, &size, NULL, NULL) == FALSE) {
+		g_object_unref (itms_file);
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 	}
-	g_free (itms_url);
+	g_object_unref (itms_file);
 
 	uncompressed = decompress_gzip (contents, size);
 	g_free (contents);
@@ -692,25 +703,39 @@ totem_pl_parser_add_itms (TotemPlParser *parser,
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	ret = totem_pl_parser_add_rss (parser, feed_url, NULL, NULL);
-	g_free (feed_url);
+	g_object_unref (feed_url);
 
 	return ret;
 }
 
 gboolean
-totem_pl_parser_is_itms_feed (const char *url)
+totem_pl_parser_is_itms_feed (GFile *file)
 {
-	g_return_val_if_fail (url != NULL, FALSE);
+	char *url;
 
-	if (g_str_has_prefix (url, "itms:") != FALSE
+	g_return_val_if_fail (file != NULL, FALSE);
+
+	url = g_file_get_uri (file);
+
+	if (g_file_has_uri_scheme (file, "itms:") != FALSE
 	    && strstr (url, "phobos.apple.com") != NULL
-	    && strstr (url, "viewPodcast") != NULL)
+	    && strstr (url, "viewPodcast") != NULL) {
+	    	g_free (url);
 		return TRUE;
+	}
+
 	if (strstr (url, "phobos.apple.com/") != NULL
-	    && strstr (url, "viewPodcast") != NULL)
+	    && strstr (url, "viewPodcast") != NULL) {
+	    	g_free (url);
 		return TRUE;
-	if (strstr (url, "itunes.com/podcast") != NULL)
+	}
+
+	if (strstr (url, "itunes.com/podcast") != NULL) {
+		g_free (url);
 		return TRUE;
+	}
+
+	g_free (url);
 
 	return FALSE;
 }
@@ -772,15 +797,15 @@ parse_opml_head_body (TotemPlParser *parser, const char *url, xml_node_t *parent
 
 TotemPlParserResult
 totem_pl_parser_add_opml (TotemPlParser *parser,
-			  const char *url,
-			  const char *base,
+			  GFile *file,
+			  GFile *base_file,
 			  gpointer data)
 {
 	xml_node_t* doc;
-	char *contents;
-	int size;
+	char *contents, *url;
+	gsize size;
 
-	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	if (g_file_load_contents (file, NULL, &contents, &size, NULL, NULL) == FALSE)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	xml_parser_init (contents, size, XML_PARSER_CASE_INSENSITIVE);
@@ -796,7 +821,9 @@ totem_pl_parser_add_opml (TotemPlParser *parser,
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 	}
 
+	url = g_file_get_uri (file);
 	parse_opml_head_body (parser, url, doc);
+	g_free (url);
 
 	g_free (contents);
 	xml_parser_free_tree (doc);
